import React from 'react';
import PropTypes from 'prop-types';
import momentPropTypes from 'react-moment-proptypes';
import moment from 'moment';
import { extendMoment } from 'moment-range';
import isTouchDevice from 'is-touch-device';
import Event from './Event';
import EventPreview from './EventPreview';
import SlotsContainer from './SlotsContainer';

const propTypes = {
  day: momentPropTypes.momentObj.isRequired,
  slotDuration: momentPropTypes.momentDurationObj.isRequired,

  hourStart: PropTypes.number,
  hourEnd: PropTypes.number,

  events: PropTypes.arrayOf(PropTypes.shape({
    id: PropTypes.number.isRequired,
    beginAt: PropTypes.string.isRequired,
    endAt: PropTypes.string.isRequired,
  })),

  onEventAdd: PropTypes.func,
  onEventDelete: PropTypes.func,
  onEventInstant: PropTypes.func,
};

const defaultProps = {
  hourStart: 0,
  hourEnd: 24,

  events: [],

  onEventAdd() {},
  onEventDelete() {},
  onEventInstant() {},
};

class Day extends React.PureComponent {
  constructor(props) {
    super(props);

    this.state = {
      slotStartFired: null,
    };

    this.isTouchDevice = false;

    this.onSlotMouseEnter = this.onSlotMouseEnter.bind(this);
    this.onSlotMouseLeave = this.onSlotMouseLeave.bind(this);
    this.onSlotMouseClick = this.onSlotMouseClick.bind(this);
    this.getYStyles = this.getYStyles.bind(this);
    this.isSlotAssignable = this.isSlotAssignable.bind(this);
    this.onEventDeleteClick = this.onEventDeleteClick.bind(this);
    this.onEventInstantClick = this.onEventInstantClick.bind(this);
  }

  componentDidMount() {
    this.isTouchDevice = isTouchDevice();
  }

  onSlotMouseEnter(slotStart) {
    if (!this.isTouchDevice) {
      this.setState({
        slotStartFired: moment(slotStart),
      });
    }
  }
  onSlotMouseLeave() {
    if (!this.isTouchDevice) {
      this.setState({
        slotStartFired: null,
      });
    }
  }
  onSlotMouseClick(slotStart, slotEnd) {
    const { onEventAdd } = this.props;
    const { slotStartFired } = this.state;
    if (slotStartFired !== null) {
      if (slotStartFired.isSame(slotStart)) {
        // We can create an event
        onEventAdd(slotStart.toISOString(), slotEnd.toISOString());

        this.setState({
          slotStartFired: null,
        });
      } else {
        this.setState({
          slotStartFired: moment(slotStart),
        });
      }
    } else {
      this.setState({
        slotStartFired: moment(slotStart),
      });
    }
  }
  onEventDeleteClick(id) {
    const { onEventDelete } = this.props;
    onEventDelete(id);
  }
  onEventInstantClick(id) {
    const { onEventInstant } = this.props;
    onEventInstant(id);
  }

  getYStyles(startTime, endTime) {
    const {
      hourStart,
      slotDuration,
    } = this.props;
    const slotHeight = 34;

    const dayMinutesStart = hourStart * 60;
    const elementMinutesStart = (moment(startTime).hour() * 60) + moment(startTime).minute();
    const elementTop =
      (elementMinutesStart - dayMinutesStart) / slotDuration.minutes() * slotHeight;

    return ({
      top: `${elementTop}px`,
      height: `${slotHeight - 5}px`,
      left: 0,
      right: 0,
    });
  }

  isSlotAssignable(slotStart, slotEnd) {
    const momentRange = extendMoment(moment);
    const { events } = this.props;
    const slotRange = momentRange.range(slotStart, slotEnd);
    const eventsInSlot = events.filter(
      event => slotRange.overlaps(momentRange.range(event.beginAt, event.endAt)),
    );
    return eventsInSlot.length === 0;
  }

  renderEvents() {
    const { events } = this.props;
    return (
      <div className="Day__events">
        {events.map(event => (
          <Event
            key={event.id}
            id={event.id}
            eventStart={event.beginAt}
            eventEnd={event.endAt}
            isInstant={event.isInstant}
            getStyle={() => this.getYStyles(event.beginAt)}
            onDeleteClick={this.onEventDeleteClick}
            onInstantClick={this.onEventInstantClick}
          />
          ))}
      </div>
    );
  }

  renderPreview() {
    const { slotStartFired } = this.state;
    if (slotStartFired !== null) {
      this.getYStyles(slotStartFired);
      return (
        <EventPreview
          getStyle={() => this.getYStyles(moment(slotStartFired))}
        />
      );
    }
    return (<div />);
  }

  render() {
    const {
      day,
      slotDuration,
      hourStart,
      hourEnd,
    } = this.props;

    return (
      <div className="Day">
        <div className="Day__header">
          {day.format('ddd D')}
        </div>
        <div className="Day__hours">
          {this.renderEvents()}
          {this.renderPreview()}
          <SlotsContainer
            day={day}
            slotDuration={slotDuration}
            hourStart={hourStart}
            hourEnd={hourEnd}
            onSlotMouseEnter={this.onSlotMouseEnter}
            onSlotMouseLeave={this.onSlotMouseLeave}
            onSlotMouseClick={this.onSlotMouseClick}
            isSlotAssignable={this.isSlotAssignable}
          />
        </div>
      </div>
    );
  }
}

Day.propTypes = propTypes;
Day.defaultProps = defaultProps;

export default Day;
