import React from 'react';
import PropTypes from 'prop-types';
import momentPropTypes from 'react-moment-proptypes';
import moment from 'moment';

import Day from './Day';
import Navigation from './Navigation';

const propTypes = {
  days: PropTypes.number,
  slotDuration: momentPropTypes.momentDurationObj,

  dayStart: PropTypes.string,
  hourStart: PropTypes.number,
  hourEnd: PropTypes.number,

  events: PropTypes.arrayOf(PropTypes.shape({
    id: PropTypes.number.isRequired,
    beginAt: PropTypes.string.isRequired,
    endAt: PropTypes.string.isRequired,
  })),

  onEventAdd: PropTypes.func,
  onEventDelete: PropTypes.func,
  onEventInstant: PropTypes.func,
  onNavigate: PropTypes.func,
};

const defaultProps = {
  days: 1,
  slotDuration: moment.duration(15, 'minutes'),

  dayStart: moment().toISOString(),
  hourStart: 0,
  hourEnd: 24,

  events: [],

  onEventAdd() {},
  onEventDelete() {},
  onEventInstant() {},
  onNavigate() {},
};

class Timetable extends React.Component {
  constructor(props) {
    super(props);

    const {
      days,
      dayStart,
    } = this.props;
    this.state = {
      dayStart: moment(dayStart),
      dayEnd: moment(dayStart).startOf('day').add(days, 'd'),
    };

    this.getDayEvents = this.getDayEvents.bind(this);
    this.onNavigatePrevious = this.onNavigatePrevious.bind(this);
    this.onNavigateNext = this.onNavigateNext.bind(this);
  }

  onNavigatePrevious() {
    const {
      days,
    } = this.props;
    const { dayStart } = this.state;
    const newDayStart = moment(dayStart).subtract(days, 'd');

    this.setState({
      dayStart: newDayStart,
      dayEnd: dayStart,
    });

    this.onNavigate(newDayStart);
  }

  onNavigateNext() {
    const {
      days,
    } = this.props;
    const { dayEnd } = this.state;
    const newDayEnd = moment(dayEnd).add(days, 'd')

    this.setState({
      dayStart: dayEnd,
      dayEnd: newDayEnd,
    });

    this.onNavigate(dayEnd);
  }

  onNavigate(dayStart) {
    const { onNavigate } = this.props;
    onNavigate(dayStart);
  }

  getDayEvents(day) {
    const { events } = this.props;
    return events.filter(event => moment(event.beginAt).isSame(day, 'day'));
  }

  renderDays() {
    const {
      slotDuration,
      hourStart,
      hourEnd,
      onEventAdd,
      onEventDelete,
      onEventInstant,
    } = this.props;
    const { dayStart, dayEnd } = this.state;
    const dayMoments = [];

    for (let i = moment(dayStart); i.isBefore(dayEnd); i.add(1, 'day')) {
      dayMoments.push(moment(i));
    }

    return (
      <div className="Timetable__days">
        {dayMoments.map(((day, key) =>
          <Day
            key={key}
            day={day}
            slotDuration={slotDuration}
            hourStart={hourStart}
            hourEnd={hourEnd}
            events={this.getDayEvents(day)}
            onEventAdd={onEventAdd}
            onEventDelete={onEventDelete}
            onEventInstant={onEventInstant}
          />
        ))}
      </div>
    );
  }
  render() {
    return (
      <div className="Timetable">
        <Navigation
          onNavigatePrevious={this.onNavigatePrevious}
          onNavigateNext={this.onNavigateNext}
        />
        {this.renderDays()}
      </div>
    );
  }
}

Timetable.propTypes = propTypes;
Timetable.defaultProps = defaultProps;

export default Timetable;
